@@ -1,9 +1,6 @@
 from MLVisualizationTools import Analytics, Interfaces, Graphs, Colorizers
-from MLVisualizationTools.backend import fileloader
+from MLVisualizationTools.dashbackend import getTheme, getDashApp
 import pandas as pd
-import os
-os.environ['TF_CPP_MIN_LOG_LEVEL'] = '2' #stops agressive error message printing
-from tensorflow import keras
 
 try:
     import dash
@@ -11,47 +8,23 @@ try:
     from dash import dcc
     from dash import html
     import dash_bootstrap_components as dbc
+    from dash_tour_component import DashTour
     import plotly
-except:
-    raise ImportError("Dash and plotly are required to run this demo. Install them with the [dash] flag"
+except ImportError:
+    raise ImportError("Dash and plotly are required to use this tool. Install them with the [dash] flag"
                       " on installation of this library.")
 
 class App:
-    def __init__(self, model, data: pd.DataFrame, title:str = "DashModelVisualizer", highcontrast:bool = True,
-                 notebook:bool = False, kagglenotebook:bool = False, mode:str = 'external',
+    def __init__(self, model, data: pd.DataFrame, title:str = "DashModelVisualizer", theme:str = "dark", folder = None,
+                 highcontrast:bool = True, notebook:bool = False, kagglenotebook:bool = False, mode:str = 'external',
                  host:str = '0.0.0.0', port: bool = None):
 
-        if port is None:
-            if notebook:
-                self.port = 1005
-            else:
-                self.port = 8050
-        else:
-            self.port = port
-
-        if notebook:
-            from jupyter_dash import JupyterDash
-            if kagglenotebook:
-                try:
-                    from pyngrok import ngrok
-                except:
-                    raise ImportError("Pyngrok is required to run in a kaggle notebook.")
-                tunnel = ngrok.connect(self.port)
-                print("Running in an ngrok tunnel. This limits you to 40 requests per minute. For full features",
-                      "use google colab instead.")
-                url = tunnel.public_url
-            else:
-                url = None
-            self.app = JupyterDash(__name__, title=title, server_url=url)
-        else:
-            self.app = dash.Dash(__name__, title=title)
+        theme, folder, self.figtemplate = getTheme(theme, folder)
+        self.app, self.runFunc = getDashApp(title, notebook, kagglenotebook, host, port, mode, theme, folder)
 
         self.model = model
         self.df = data
         self.highcontrast = highcontrast
-        self.notebook = notebook
-        self.mode = mode
-        self.host = host
 
         options = []
         for col in self.df.columns:
@@ -76,27 +49,50 @@ class App:
             dbc.Label("Y Axis: "),
             dcc.Dropdown(id='yaxis', options=options, value=self.y),
             html.Br(),
+            dbc.Button("Open Tour", id='open_tour_button'),
+            html.Br(),
         ], body=True)
 
+        tour = DashTour(steps=[{'selector': '[id="xaxis"]',
+                                'content': "This dropdown controls the xaxis, it has been preset with the "
+                                           "highest variance value."},
+                               {'selector': '[id="yaxis"]',
+                                'content': "This dropdown controls the yaxis, it has been preset with the "
+                                           "second highest variance value."},
+                               {'selector': '[id="example-graph"]',
+                                'content': "This graph updates when you change the dropdowns."}],
+                        isOpen=False,
+                        id="tour_component",
+                        children=html.Div(),
+                        rounded=7
+                        )
+
         self.app.layout = dbc.Container([
+            tour,
             html.H1(title),
             html.Hr(),
             dbc.Row([
                 dbc.Col(config, md=4),
                 dbc.Col(graph, md=8)]
             ),
-            html.P()])
+            html.P()],
+            fluid=True,
+            className='dash-bootstrap'
+        )
 
         inputs = [Input('xaxis', "value"), Input('yaxis', 'value')]
         self.app.callback(Output("example-graph", "figure"), inputs)(self.updateGraphFromWebsite)
+        self.app.callback(Output('tour_component', 'isOpen'), [Input('open_tour_button', 'n_clicks')],
+                          prevent_initial_call=True)(lambda _ : True)
 
     def run(self):
-        self.app.run_server(host = self.host, port = self.port, mode=self.mode)
+        self.runFunc()
 
     def updateGraph(self):
         data = Interfaces.TensorflowGrid(self.model, self.x, self.y, self.df)
         data = Colorizers.Binary(data, highcontrast=self.highcontrast)
         self.fig = Graphs.PlotlyGrid(data, self.x, self.y)
+        self.fig.update_layout(template=self.figtemplate)
         return self.fig
 
     def updateGraphFromWebsite(self, x, y):
@@ -104,14 +100,17 @@ class App:
         self.y = y
         return self.updateGraph()
 
-def main(model, data: pd.DataFrame, title:str = "DashModelVisualizer", highcontrast:bool = True, notebook:bool = False,
-         kagglenotebook:bool = False, mode:str = 'external', host:str = '0.0.0.0', port: bool = None):
+def visualize(model, data: pd.DataFrame, title:str = "DashModelVisualizer", theme:str = "dark", folder = None,
+              highcontrast:bool = True, notebook:bool = False, kagglenotebook:bool = False, mode:str = 'external',
+              host:str = '0.0.0.0', port: bool = None):
     """
     Creates a dash website to visualize an ML model.
 
     :param model: A tensorflow keras model
     :param data: A pandas dataframe, all df columns must be numerical model inputs
     :param title: Title for website
+    :param theme: Theme to load app in, can be a string (light / dark) or a url to load a stylesheet from
+    :param folder: Directory to load additional css and js from
     :param highcontrast: Visualizes the model with orange and blue instead of green and red. Great for colorblind people!
     :param notebook: Uses jupyter dash instead of dash
     :param kagglenotebook: Enables ngrok tunneling for use in kaggle notebooks
@@ -119,14 +118,4 @@ def main(model, data: pd.DataFrame, title:str = "DashModelVisualizer", highcontr
     :param host: default hostname for dash
     :param port: None for default port (8050) or (1005)
     """
-
-    App(model, data, title, highcontrast, notebook, kagglenotebook, mode, host, port).run()
-
-def default():
-    model = keras.models.load_model(fileloader('examples/Models/titanicmodel'))
-    df: pd.DataFrame = pd.read_csv(fileloader('examples/Datasets/Titanic/train.csv'))
-    df = df.drop('Survived', axis=1)
-    main(model, df)
-
-if __name__ == '__main__':
-    default()
\ No newline at end of file
+    App(model, data, title, theme, folder, highcontrast, notebook, kagglenotebook, mode, host, port).run()
