@@ -1,5 +1,5 @@
 from typing import List, Dict
-from MLVisualizationTools.backend import colinfo
+from MLVisualizationTools.backend import colinfo, GraphData, GraphDataTypes
 import pandas as pd
 
 #Functions for passing data to ml models
@@ -7,7 +7,7 @@ import pandas as pd
 #region Tensorflow
 #region grid
 def TFModelPredictionGrid(model, x:str, y:str, data:pd.DataFrame,
-                          exclude:List[str] = None, steps:int=20) -> pd.DataFrame:
+                          exclude:List[str] = None, steps:int=20) -> GraphData:
     """
     Creates a dataset from a 2d prediction on a tensorflow model. Wrapper function for TFModelPredictionGridRaw()
     that automatically handles column info generation.
@@ -21,7 +21,7 @@ def TFModelPredictionGrid(model, x:str, y:str, data:pd.DataFrame,
     """
     return TFModelPredictionGridRaw(model, x, y, colinfo(data, exclude), steps)
 
-def TFModelPredictionGridRaw(model, x:str, y:str, coldata:List[Dict], steps:int=20) -> pd.DataFrame:
+def TFModelPredictionGridRaw(model, x:str, y:str, coldata:List[Dict], steps:int=20) -> GraphData:
     """
     Creates a dataset from a 2d prediction on a tensorflow model. Wrapper function for TFModelPredictionGridRaw()
     that automatically handles column info generation.
@@ -37,36 +37,50 @@ def TFModelPredictionGridRaw(model, x:str, y:str, coldata:List[Dict], steps:int=
     :param coldata: An ordered list of dicts with col names, min max values, and means
     :param steps: Resolution to scan model with
     """
+    allcols = []
+    for item in coldata:
+        allcols.append(item['name'])
+
+    assert x in allcols, "X must be in coldata"
+    assert y in allcols, "Y must be in coldata"
+
     cols = []
     for item in coldata:
-        cols.append(item['name'])
-    preddata = pd.DataFrame(columns=cols)
+        if item not in [x, y]:
+            cols.append(item['name'])
 
-    assert x in cols, "X must be in coldata"
-    assert y in cols, "Y must be in coldata"
+    srow = []
+    for item in cols:
+        for d in coldata:
+            if d['name'] == item:
+                srow.append(d['mean'])
 
-    for xpos in range(0, steps):
-        for ypos in range(0, steps):
-            predictionrow = {}
-            for item in coldata:
-                key = item['name']
-                if key == x:
-                    predictionrow[x] = xpos * (item['max'] - item['min']) / (steps - 1) + item['min']
-                elif key == y:
-                    predictionrow[y] = ypos * (item['max'] - item['min']) / (steps - 1) + item['min']
-                else:
-                    predictionrow[key] = item['mean']
+    srow = [srow] * (steps ** 2)
+    preddata = pd.DataFrame(srow, columns=cols)
 
-            preddata = preddata.append(predictionrow, ignore_index=True)
+    col = []
+    for pos in range(0, steps):
+        for item in coldata:
+            if item['name'] == x:
+                col.append(pos * (item['max'] - item['min']) / (steps - 1) + item['min'])
+    col = col * steps
+    preddata[x] = col
+
+    col = []
+    for pos in range(0, steps):
+        for item in coldata:
+            if item['name'] == y:
+                col += [pos * (item['max'] - item['min']) / (steps - 1) + item['min']] * steps
+    preddata[y] = col
 
     predictions = model.predict(preddata)
     preddata['Output'] = predictions
-    return preddata
+    return GraphData(preddata, GraphDataTypes.Grid)
 #endregion grid
 
 #region animation
 def TFModelPredictionAnimation(model, x:str, y:str, anim:str, data: pd.DataFrame,
-                               exclude:list[str] = None, steps:int=20) -> pd.DataFrame:
+                               exclude:List[str] = None, steps:int=20) -> GraphData:
     """
     Creates a dataset from a 2d prediction on a tensorflow model. Wrapper function for TFModelPredictionGridRaw()
     that automatically handles column info generation.
@@ -81,7 +95,7 @@ def TFModelPredictionAnimation(model, x:str, y:str, anim:str, data: pd.DataFrame
     """
     return TFModelPredictionAnimationRaw(model, x, y, anim, colinfo(data, exclude), steps)
 
-def TFModelPredictionAnimationRaw(model, x:str, y:str, anim:str, coldata:List[Dict], steps:int=20) -> pd.DataFrame:
+def TFModelPredictionAnimationRaw(model, x:str, y:str, anim:str, coldata:List[Dict], steps:int=20) -> GraphData:
     """
     Creates a dataset from a 2d prediction on a tensorflow model. Wrapper function for TFModelPredictionGridRaw()
     that automatically handles column info generation.
@@ -98,34 +112,55 @@ def TFModelPredictionAnimationRaw(model, x:str, y:str, anim:str, coldata:List[Di
     :param coldata: An ordered list of dicts with col names, min max values, and means
     :param steps: Resolution to scan model with
     """
+
+    allcols = []
+    for item in coldata:
+        allcols.append(item['name'])
+
+    assert x in allcols, "X must be in coldata"
+    assert y in allcols, "Y must be in coldata"
+    assert anim in allcols, "Anim must be in coldata"
+
     cols = []
     for item in coldata:
-        cols.append(item['name'])
-    preddata = pd.DataFrame(columns=cols)
+        if item not in [x, y, anim]:
+            cols.append(item['name'])
 
-    assert x in cols, "X must be in coldata"
-    assert y in cols, "Y must be in coldata"
-    assert anim in cols, "Anim must be in coldata"
+    srow = []
+    for item in cols:
+        for d in coldata:
+            if d['name'] == item:
+                srow.append(d['mean'])
 
-    for xpos in range(0, steps):
-        for ypos in range(0, steps):
-            for animpos in range(0, steps):
-                predictionrow = {}
-                for item in coldata:
-                    key = item['name']
-                    if key == x:
-                        predictionrow[x] = xpos * (item['max'] - item['min']) / (steps - 1) + item['min']
-                    elif key == y:
-                        predictionrow[y] = ypos * (item['max'] - item['min']) / (steps - 1) + item['min']
-                    elif key == anim:
-                        predictionrow[anim] = animpos * (item['max'] - item['min']) / (steps - 1) + item['min']
-                    else:
-                        predictionrow[key] = item['mean']
+    srow = [srow] * (steps ** 3)
+    preddata = pd.DataFrame(srow, columns=cols)
 
-                preddata = preddata.append(predictionrow, ignore_index=True)
+    col = []
+    for pos in range(0, steps):
+        for item in coldata:
+            if item['name'] == x:
+                col.append(pos * (item['max'] - item['min']) / (steps - 1) + item['min'])
+    col = col * (steps ** 2)
+    preddata[x] = col
+
+    col = []
+    for pos in range(0, steps):
+        for item in coldata:
+            if item['name'] == y:
+                col += [pos * (item['max'] - item['min']) / (steps - 1) + item['min']] * steps
+    col = col * steps
+    preddata[y] = col
+
+    col = []
+    for pos in range(0, steps):
+        for item in coldata:
+            if item['name'] == anim:
+                col += [pos * (item['max'] - item['min']) / (steps - 1) + item['min']] * (steps ** 2)
+    preddata[anim] = col
 
     predictions = model.predict(preddata)
     preddata['Output'] = predictions
-    return preddata
+    return GraphData(preddata, GraphDataTypes.Animation)
+
+#endregion
 #endregion
-#endregion
\ No newline at end of file
